from typing import Optional

from pydantic import AnyHttpUrl, Field, constr, validator

from infobip_channels.core.models import MessageBodyBase, UrlLengthValidatorMixin


class MessageBody(UrlLengthValidatorMixin, MessageBodyBase):
    from_number: constr(min_length=1, max_length=24) = Field(alias="from")
    to: constr(min_length=1, max_length=24)
    message_id: Optional[constr(max_length=50)] = None
    callback_data: Optional[constr(max_length=4000)] = None
    notify_url: Optional[AnyHttpUrl] = None

    @validator("notify_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)



from abc import ABC, abstractmethod
from typing import Any, Dict, Type, Union

import requests
from pydantic import AnyHttpUrl, BaseModel, ValidationError

from infobip_channels.core.http_client import _HttpClient
from infobip_channels.core.models import (
    Authentication,
    GetHeaders,
    MessageBodyBase,
    PostHeaders,
    ResponseBase,
)
from infobip_channels.web_rtc.models.path_parameters.core import PathParameter


class Channel(ABC):
    """Parent class for all Infobip channels."""

    def __init__(self, client: Union[_HttpClient, Any]) -> None:
        self._client = client

    @classmethod
    def from_auth_params(cls, auth_params: Dict[str, str]) -> "Channel":
        """Create an Authentication instance from the provided dictionary and
        use it to instantiate the Channel subclass. Dictionary has to contain
        "base_url" and "api_key" to be able to authenticate with the Infobip's API.
        The Channel subclass instantiated this way will use the default _HttpClient
        class for making HTTP requests.

        :param auth_params: Dictionary containing "base_url" and "api_key"
        :return: Instance of the subclass
        """
        client = _HttpClient(Authentication(**auth_params))
        return cls(client)

    @classmethod
    def from_auth_instance(cls, auth_instance: Authentication) -> "Channel":
        """Instantiate the Channel subclass with the provided auth object.
        The Channel subclass instantiated this way will use the default _HttpClient
        class for making HTTP requests.

        :param auth_instance: Authentication class instance
        :return: Instance of the subclass
        """
        client = _HttpClient(auth_instance)
        return cls(client)

    @classmethod
    def from_provided_client(cls, client: Any) -> "Channel":
        """Instantiate the Channel subclass with the provided client object.
        The Channel subclass instantiated this way will use the provided client for
        making HTTP requests. This client can implement its own retry mechanisms,
        timeouts, etc., but it has to implement all the methods used in the default
        _HttpClient class. When using the Channel subclass this way, the user has to
        take care of providing a valid base_url and constructing headers to be used for
        every Channel subclass request.

        :param client: Client used for making HTTP requests
        :return: Instance of the subclass
        """
        return cls(client)

    @staticmethod
    def validate_auth_params(
        base_url: Union[AnyHttpUrl, str], api_key: str
    ) -> Authentication:
        """Validate the provided base_url and api_key. This validation is purely client
        side. If the parameters are validated successfully, an instance of the
        Authentication class is returned which holds the base_url and api_key values.

        :param base_url: Base url which the requests will call for each endpoint
        :param api_key: Secret used for authenticating the user
        :return: Authentication class instance
        """
        return Authentication(base_url=base_url, api_key=api_key)

    @staticmethod
    def build_post_request_headers(api_key: str) -> Dict:
        """Build the request headers dictionary which has to be used for each of the
        post requests.

        :param api_key: Key used for populating Authorization header
        :return: Dictionary of headers to be used for post requests
        """
        return PostHeaders(authorization=api_key).dict(by_alias=True)

    @staticmethod
    def build_get_request_headers(api_key: str) -> Dict:
        """Build the request headers dictionary which has to be used for each of the
        get requests.

        :param api_key: Key used for populating Authorization header
        :return: Dictionary of headers to be used for get requests
        """
        return GetHeaders(authorization=api_key).dict(by_alias=True)

    @staticmethod
    def convert_model_to_dict(
        model: BaseModel, by_alias: bool = True, exclude_unset: bool = True, **kwargs
    ) -> Dict:
        """
        Convert the Pydantic model into a Python dictionary. By default, model is
        converted with by_alias=True and exclude_unset=True flags. The former changes
        model fields to camel case and the latter omits the fields which were not
        received from the server originally.
        For additional flags, check Pydantic's documentation on exporting models:
        https://pydantic-docs.helpmanual.io/usage/exporting_models/.

        :param model: Pydantic model to convert
        :param by_alias: Whether the model should be converted with aliased fields
        :param exclude_unset: Whether the model's unset values should be omitted
        :return: Dictionary of the converted model
        """
        return model.dict(by_alias=by_alias, exclude_unset=exclude_unset, **kwargs)

    @staticmethod
    def validate_path_parameter(
        parameter: Union[PathParameter, Dict], parameter_type: Type[PathParameter]
    ) -> PathParameter:
        """
        Validate path parameter by trying to instantiate the provided class and
        extract valid path parameter.

        :param parameter: Path parameter to validate
        :param parameter_type: Type of path parameter
        :return: Class instance corresponding to the provided parameter type
        """
        return (
            parameter
            if isinstance(parameter, parameter_type)
            else parameter_type(**parameter)
        )

    @staticmethod
    def validate_message_body(
        message: Union[MessageBodyBase, Dict],
        message_type: Type[MessageBodyBase],
    ) -> Union[MessageBodyBase]:
        """Validate the message by trying to instantiate the provided type class.
        If the passed message is already of that type, just return it as is.

        :param message: Message body to validate
        :param message_type: Type of the message body
        :return: Class instance corresponding to the provided message body type
        """
        return message if isinstance(message, message_type) else message_type(**message)

    def _construct_response(
        self, raw_response: Union[requests.Response, Any], *args, **kwargs
    ) -> Union[ResponseBase, requests.Response, Any]:
        try:
            response_class = self._get_custom_response_class(
                raw_response, *args, **kwargs
            )

            response_json = raw_response.json()
            if type(response_json) is list:
                raw_response_data = {"list": response_json}
            else:
                raw_response_data = response_json

            return response_class(
                **{
                    "status_code": raw_response.status_code,
                    "raw_response": raw_response,
                    **raw_response_data,
                }
            )

        except (AttributeError, ValueError, ValidationError):
            return raw_response

    @abstractmethod
    def _get_custom_response_class(
        self, raw_response: Union[requests.Response, Any], *args, **kwargs
    ) -> Type[ResponseBase]:
        raise NotImplementedError



from enum import Enum
from typing import List, Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from pydantic import AnyHttpUrl, conlist, constr, validator

from infobip_channels.core.models import (
    CamelCaseModel,
    MessageBodyBase,
    UrlLengthValidatorMixin,
)


class LanguageEnum(str, Enum):
    AF = "af"
    SQ = "sq"
    AR = "ar"
    AZ = "az"
    BN = "bn"
    BG = "bg"
    CA = "ca"
    ZH_CN = "zh_CN"
    ZH_HK = "zh_HK"
    ZH_TW = "zh_TW"
    HR = "hr"
    CS = "cs"
    DA = "da"
    NL = "nl"
    EN = "en"
    EN_GB = "en_GB"
    EN_US = "en_US"
    ET = "et"
    FIL = "fil"
    FI = "fi"
    FR = "fr"
    DE = "de"
    EL = "el"
    GU = "gu"
    HA = "ha"
    HE = "he"
    HI = "hi"
    HU = "hu"
    ID = "id"
    GA = "ga"
    IT = "it"
    JA = "ja"
    KN = "kn"
    KK = "kk"
    KO = "ko"
    LO = "lo"
    LV = "lv"
    LT = "lt"
    MK = "mk"
    MS = "ms"
    ML = "ml"
    MR = "mr"
    NB = "nb"
    FA = "fa"
    PL = "pl"
    PT_BR = "pt_BR"
    PT_PT = "pt_PT"
    PA = "pa"
    RO = "ro"
    RU = "ru"
    SR = "sr"
    SK = "sk"
    SL = "sl"
    ES = "es"
    ES_AR = "es_AR"
    ES_ES = "es_ES"
    ES_MX = "es_MX"
    SW = "sw"
    SV = "sv"
    TA = "ta"
    TE = "te"
    TH = "th"
    TR = "tr"
    UK = "uk"
    UR = "ur"
    UZ = "uz"
    VI = "vi"
    UNKNOWN = "unknown"


class HeaderText(CamelCaseModel):
    format: Literal["TEXT"]
    text: constr(max_length=60)


class HeaderImage(CamelCaseModel):
    format: Literal["IMAGE"]


class HeaderVideo(CamelCaseModel):
    format: Literal["VIDEO"]


class HeaderDocument(CamelCaseModel):
    format: Literal["DOCUMENT"]


class HeaderLocation(CamelCaseModel):
    format: Literal["LOCATION"]


class CategoryEnum(str, Enum):
    ACCOUNT_UPDATE = "ACCOUNT_UPDATE"
    PAYMENT_UPDATE = "PAYMENT_UPDATE"
    PERSONAL_FINANCE_UPDATE = "PERSONAL_FINANCE_UPDATE"
    SHIPPING_UPDATE = "SHIPPING_UPDATE"
    RESERVATION_UPDATE = "RESERVATION_UPDATE"
    ISSUE_RESOLUTION = "ISSUE_RESOLUTION"
    APPOINTMENT_UPDATE = "APPOINTMENT_UPDATE"
    TRANSPORTATION_UPDATE = "TRANSPORTATION_UPDATE"
    TICKET_UPDATE = "TICKET_UPDATE"
    ALERT_UPDATE = "ALERT_UPDATE"
    AUTO_REPLY = "AUTO_REPLY"


class Button(CamelCaseModel):
    text: constr(max_length=200)


class ButtonPhoneNumber(Button):
    type: Literal["PHONE_NUMBER"]
    phone_number: str


class ButtonUrl(UrlLengthValidatorMixin, Button):
    type: Literal["URL"]
    url: AnyHttpUrl

    @validator("url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class ButtonQuickReply(Button):
    type: Literal["QUICK_REPLY"]


class Structure(CamelCaseModel):
    header: Optional[
        Union[HeaderText, HeaderImage, HeaderVideo, HeaderDocument, HeaderLocation]
    ] = None
    body: str
    footer: Optional[constr(max_length=60)] = None
    buttons: Optional[
        Union[
            conlist(Union[ButtonPhoneNumber, ButtonUrl], max_items=2),
            conlist(ButtonQuickReply, max_items=3),
        ]
    ] = None

    @validator("buttons")
    def validate_buttons(cls, buttons: List[Button]) -> List[Button]:
        if not buttons or isinstance(buttons[0], ButtonQuickReply) or len(buttons) == 1:
            return buttons

        if buttons[0].type == buttons[1].type:
            raise ValueError("Call to action buttons must be unique in type")

        return buttons


class CreateTemplate(MessageBodyBase):
    name: constr(regex=r"^[a-z0-9_]+$")  # noqa: F722
    language: LanguageEnum
    category: CategoryEnum
    structure: Structure



from enum import Enum
from typing import Optional

import regex
from pydantic import conint, constr, validator

from infobip_channels.core.models import CamelCaseModel, MessageBodyBase


class RecordingEnum(str, Enum):
    ALWAYS = "ALWAYS"
    ON_DEMAND = "ON_DEMAND"
    DISABLED = "DISABLED"


class Capabilities(CamelCaseModel):
    recording: RecordingEnum


class GenerateTokenBody(MessageBodyBase):
    identity: str
    application_id: Optional[str] = None
    display_name: Optional[constr(min_length=5, max_length=50)] = None
    capabilities: Optional[Capabilities] = None
    time_to_live: Optional[conint(gt=0, lt=86401)] = None

    @validator("identity")
    def validate_identity(cls, identity: str) -> str:
        """
        Using regex lib for unicode characters
        This module supports Unicode 14.0.0.
        Full Unicode case-folding is supported.
        https://pypi.org/project/regex/

        :param identity: Identity string to check
        :return: Valid Identity string format
        """
        identity_match = regex.match(r"^[\p{L}\p{N}\-_+=/.]{3,64}$", identity)
        if identity_match:
            return identity
        else:
            raise ValueError("Identity format is not valid")



from typing import List, Optional

from pydantic import Field

from infobip_channels.core.models import CamelCaseModel, ResponseBase, ResponseStatus
from infobip_channels.mms.models.response.core import Price


class Error(CamelCaseModel):
    group_id: Optional[int] = None
    group_name: Optional[str] = None
    id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None


class Result(CamelCaseModel):
    bulk_id: Optional[str] = None
    message_id: Optional[str] = None
    to: Optional[str] = None
    from_id: Optional[str] = Field(default=None, alias="from")
    sent_at: Optional[str] = None
    done_at: Optional[str] = None
    mms_count: Optional[int] = None
    mcc_mnc: Optional[str] = None
    callback_data: Optional[str] = None
    price: Optional[Price] = None
    status: Optional[ResponseStatus] = None
    error: Optional[Error] = None


class GetMMSDeliveryReportsResponse(ResponseBase):
    results: Optional[List[Result]] = None



from typing import List, Optional

from infobip_channels.core.models import ResponseBase
from infobip_channels.whatsapp.models.response.core import WhatsAppResponseOKPayload


class TemplateMessageResponseOK(ResponseBase):
    messages: List[WhatsAppResponseOKPayload]
    bulk_id: Optional[str] = None



from typing import List, Optional

from pydantic import constr

from infobip_channels.core.models import CamelCaseModel


class SectionBase(CamelCaseModel):
    title: Optional[constr(max_length=24)] = None


class SectionTitleValidatorMixin:
    @classmethod
    def validate_section_titles(cls, sections: List[SectionBase]) -> None:
        if len(sections) > 1:
            for section in sections:
                if not section.title:
                    raise ValueError(
                        "When there is more than one section, "
                        "each one of them needs to have a title"
                    )



import json
import os
import xml.etree.ElementTree as ET
from http import HTTPStatus
from io import IOBase
from typing import Any, Dict, List, Optional, Tuple, Union

import requests
from pydantic import AnyHttpUrl, BaseModel, constr, validator
from urllib3 import encode_multipart_formdata


def to_camel_case(string: str) -> str:
    output = "".join(word.capitalize() for word in string.split("_"))
    return output[0].lower() + output[1:]


def to_header_specific_case(string: str) -> str:
    return "-".join(word.capitalize() for word in string.split("_"))


class Authentication(BaseModel):
    base_url: AnyHttpUrl
    api_key: constr(min_length=1)

    @validator("base_url", pre=True)
    def validate_scheme(cls, value: str) -> str:
        if not isinstance(value, str):
            return value

        if value.startswith(("http://", "https://")):
            return value

        return f"https://{value}"


class UrlLengthValidatorMixin:
    MAX_URL_LENGTH = 2048

    @classmethod
    def validate_url_length(cls, value: str) -> str:
        if not isinstance(value, str):
            return value

        if len(value) > cls.MAX_URL_LENGTH:
            raise ValueError(f"Url length must be less than {cls.MAX_URL_LENGTH}")

        return value


class CamelCaseModel(BaseModel):
    class Config:
        alias_generator = to_camel_case
        allow_population_by_field_name = True


class MessageBodyBase(CamelCaseModel):
    pass


class ResponseBase(CamelCaseModel):
    status_code: HTTPStatus
    raw_response: requests.Response

    class Config(CamelCaseModel.Config):
        arbitrary_types_allowed = True


class ResponseStatus(CamelCaseModel):
    group_id: int
    group_name: str
    id: int
    name: str
    description: str
    action: Optional[str] = None


class RequestHeaders(BaseModel):
    authorization: str
    accept: Optional[str] = "application/json"

    class Config:
        alias_generator = to_header_specific_case
        allow_population_by_field_name = True

    def __init__(self, **data: str) -> None:
        super().__init__(**data)
        self.authorization = f"App {self.authorization}"


class GetHeaders(RequestHeaders):
    pass


class PostHeaders(RequestHeaders):
    content_type: Optional[str] = "application/json"


class PutHeaders(RequestHeaders):
    content_type: Optional[str] = "application/json"


class DeleteHeaders(RequestHeaders):
    pass


class PathParameter(CamelCaseModel):
    sender: str


class QueryParameter(CamelCaseModel):
    pass


class XML(str):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, value):
        if not isinstance(value, str):
            raise TypeError("String required")

        if not value:
            return

        try:
            ET.fromstring(value)
        except ET.ParseError:
            raise ValueError("Invalid XML string sent")

        return cls(value)


class MultipartMixin:
    """Mixin used for allowing models to export their fields to a multipart/form-data
    format. Field types currently supported are listed in the
    _FIELD_TYPE_TO_MULTIPART_INFO_MAP attribute. All other BaseModel type fields are
    covered with the _JSON_INFO attribute.
    """

    _FIELD_TYPE_TO_MULTIPART_INFO_MAP: Dict = {
        str: {"is_file": False, "content_type": "text/plain"},
        IOBase: {"is_file": True, "content_type": ""},
        XML: {"is_file": False, "content_type": "application/xml"},
    }

    _JSON_INFO: Dict = {"is_file": False, "content_type": "application/json"}

    def to_multipart(self) -> Tuple[bytes, str]:
        """Export model's fields to a multipart/form-data format. The method returns
        a tuple of binary type body to send via POST request, and the content_type
        of the body including the multipart boundary.
        The resulting multipart_fields dictionary will have the model's field names as
        its keys and the tuples describing fields' multipart format as their values.

        For a Pydantic model with a "name: str" field and an "address: Address" field,
        the resulting multipart_fields dictionary would look like:

         multipart_fields = {
            "name": (None, "some name", "text/plain"),
            "address:" (None, {"street": "...", "city": "..."}, "application/json")
        }

        The tuple format is enforced by the urllib3's encode_multipart_formdata
        function: (filename, data, MIME type), where the MIME type is optional.
        The filename is None for all non-file types.

        :return: Tuple of binary encoded body and the multipart's content_type
        """
        multipart_fields = {}

        for field_name, field_object in self.__fields__.items():
            self._add_multipart_tuple(
                multipart_fields,
                to_camel_case(field_name),
                getattr(self, field_name),
                field_object.type_,
            )

        return encode_multipart_formdata(multipart_fields)

    def _add_multipart_tuple(
        self, multipart_fields: Dict, field_name: str, field_value: Any, field_type: Any
    ) -> None:
        if not field_value:
            return

        field_info = self._FIELD_TYPE_TO_MULTIPART_INFO_MAP.get(
            field_type, self._JSON_INFO
        )
        multipart_fields[field_name] = self._get_multipart_tuple(
            field_value, field_info
        )

    def _get_multipart_tuple(self, field_value: Any, field_info: Dict) -> Tuple:
        if field_info["is_file"]:
            return os.path.basename(field_value.name), field_value.read()

        if field_info["content_type"] == "application/json":
            field_value = self._get_json_for_field(field_value)

        return None, field_value, field_info["content_type"]

    @staticmethod
    def _get_json_for_field(model: Union[CamelCaseModel, List[CamelCaseModel]]) -> str:
        if isinstance(model, list):
            model_aliased = [item.dict(by_alias=True) for item in model]
        else:
            model_aliased = model.dict(by_alias=True)

        return json.dumps(model_aliased)



from typing import List, Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from pydantic import AnyHttpUrl, Field, confloat, conlist, constr, validator

from infobip_channels.core.models import (
    CamelCaseModel,
    MessageBodyBase,
    UrlLengthValidatorMixin,
)
from infobip_channels.whatsapp.models.body.core import MessageBody


class ButtonQuickReply(CamelCaseModel):
    type: Literal["QUICK_REPLY"]
    parameter: constr(min_length=1, max_length=128)


class ButtonUrl(CamelCaseModel):
    type: Literal["URL"]
    parameter: str


class HeaderLocation(CamelCaseModel):
    type: Literal["LOCATION"]
    latitude: confloat(ge=-90, le=90)
    longitude: confloat(ge=-180, le=180)


class HeaderDocument(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["DOCUMENT"]
    media_url: AnyHttpUrl
    filename: constr(min_length=1, max_length=240)

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderVideo(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["VIDEO"]
    media_url: AnyHttpUrl

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderImage(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["IMAGE"]
    media_url: AnyHttpUrl

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderText(CamelCaseModel):
    type: Literal["TEXT"]
    placeholder: str


class Placeholders(CamelCaseModel):
    placeholders: List[str]

    @validator("placeholders", pre=True)
    def validate_placeholders(cls, placeholders: List[str]) -> List[str]:
        for placeholder in placeholders:
            if not placeholder:
                raise ValueError("Placeholder value must not be None or empty")

        return placeholders


class TemplateData(CamelCaseModel):
    body: Placeholders
    header: Optional[
        Union[HeaderText, HeaderImage, HeaderDocument, HeaderVideo, HeaderLocation]
    ] = None
    buttons: Optional[
        Union[conlist(ButtonUrl, max_items=1), conlist(ButtonQuickReply, max_items=3)]
    ] = None


class Content(CamelCaseModel):
    template_name: constr(
        min_length=1, max_length=512, regex=r"^[a-z0-9_]+$"  # noqa: F722
    )
    template_data: TemplateData
    language: constr(min_length=1)


class SmsFailover(CamelCaseModel):
    from_number: constr(min_length=1, max_length=24) = Field(alias="from")
    text: constr(min_length=1, max_length=4096)


class Message(MessageBody):
    content: Content
    sms_failover: Optional[SmsFailover] = None


class TemplateMessageBody(MessageBodyBase):
    messages: List[Message]
    bulk_id: Optional[constr(max_length=100)] = None



from enum import Enum
from typing import List, Optional

from pydantic import conlist, constr, validator

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.action_sections import (
    SectionBase,
    SectionTitleValidatorMixin,
)
from infobip_channels.whatsapp.models.body.core import MessageBody

MESSAGE_ROWS_MAXIMUM_NUMBER = 10


class HeaderTypeEnum(str, Enum):
    TEXT = "TEXT"


class Footer(CamelCaseModel):
    text: constr(min_length=1, max_length=60)


class Header(CamelCaseModel):
    type: HeaderTypeEnum
    text: constr(min_length=1, max_length=60)


class Row(CamelCaseModel):
    id: constr(min_length=1, max_length=200)
    title: constr(min_length=1, max_length=24)
    description: Optional[constr(max_length=72)] = None


class Section(SectionBase):
    title: Optional[constr(max_length=24)] = None
    rows: conlist(Row, min_items=1)


class Action(SectionTitleValidatorMixin, CamelCaseModel):
    title: constr(min_length=1, max_length=20)
    sections: conlist(Section, min_items=1, max_items=10)

    @validator("sections")
    def validate_sections(cls, sections: List[Section]) -> List[SectionBase]:
        super().validate_section_titles(sections)
        cls._validate_row_id_uniqueness(sections)
        cls._validate_number_of_rows(sections)

        return sections

    @classmethod
    def _validate_row_id_uniqueness(cls, sections: List[Section]) -> None:
        row_ids = []
        for section in sections:
            for row in section.rows:
                if row.id in row_ids:
                    raise ValueError("Row ids must be unique across all sections")
                row_ids.append(row.id)

    @classmethod
    def _validate_number_of_rows(cls, sections: List[Section]) -> None:
        number_of_rows = 0
        for section in sections:
            number_of_rows += len(section.rows)

        if number_of_rows > MESSAGE_ROWS_MAXIMUM_NUMBER:
            raise ValueError(
                f"Message must have a maximum of {MESSAGE_ROWS_MAXIMUM_NUMBER} rows"
            )


class Body(CamelCaseModel):
    text: constr(min_length=1, max_length=1024)


class Content(CamelCaseModel):
    body: Body
    action: Action
    header: Optional[Header] = None
    footer: Optional[Footer] = None


class ListMessageBody(MessageBody):
    content: Content



from typing import Dict, List, Optional

from infobip_channels.core.models import CamelCaseModel, ResponseBase, ResponseStatus


class ServiceException(CamelCaseModel):
    message_id: str
    text: str
    validation_errors: Optional[Dict[str, List[str]]] = None


class RequestError(CamelCaseModel):
    service_exception: ServiceException


class WhatsAppResponseError(ResponseBase):
    request_error: RequestError


class WhatsAppResponseOKPayload(CamelCaseModel):
    to: str
    message_count: int
    message_id: str
    status: ResponseStatus


class WhatsAppResponseOK(WhatsAppResponseOKPayload, ResponseBase):
    pass



from datetime import date
from enum import Enum
from typing import List, Optional

from pydantic import validator

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.core import MessageBody


class AddressTypeEnum(str, Enum):
    HOME = "HOME"
    WORK = "WORK"


class EmailTypeEnum(str, Enum):
    HOME = "HOME"
    WORK = "WORK"


class UrlTypeEnum(str, Enum):
    HOME = "HOME"
    WORK = "WORK"


class PhoneTypeEnum(str, Enum):
    CELL = "CELL"
    MAIN = "MAIN"
    IPHONE = "IPHONE"
    HOME = "HOME"
    WORK = "WORK"


class Address(CamelCaseModel):
    street: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    zip: Optional[str] = None
    country: Optional[str] = None
    country_code: Optional[str] = None
    type: Optional[AddressTypeEnum]


class Email(CamelCaseModel):
    email: Optional[str] = None
    type: Optional[EmailTypeEnum]


class Name(CamelCaseModel):
    first_name: str
    last_name: Optional[str] = None
    middle_name: Optional[str] = None
    name_suffix: Optional[str] = None
    name_prefix: Optional[str] = None
    formatted_name: str


class Org(CamelCaseModel):
    company: Optional[str] = None
    department: Optional[str] = None
    title: Optional[str] = None


class Phone(CamelCaseModel):
    phone: Optional[str] = None
    type: Optional[PhoneTypeEnum]
    wa_id: Optional[str] = None


class Url(CamelCaseModel):
    url: Optional[str] = None
    type: Optional[UrlTypeEnum]


class Contact(CamelCaseModel):
    addresses: Optional[List[Address]] = None
    birthday: Optional[date] = None
    emails: Optional[List[Email]] = None
    name: Name
    org: Optional[Org] = None
    phones: Optional[List[Phone]] = None
    urls: Optional[List[Url]] = None

    @validator("birthday")
    def datetime_to_string(cls, v):
        if v:
            return v.isoformat()


class Content(CamelCaseModel):
    contacts: List[Contact]


class ContactMessageBody(MessageBody):
    content: Content



from typing import List, Optional

from pydantic import Field

from infobip_channels.core.models import CamelCaseModel, ResponseBase
from infobip_channels.mms.models.response.core import Price


class Result(CamelCaseModel):
    message_id: Optional[str] = None
    to: Optional[str] = None
    from_id: Optional[str] = Field(default=None, alias="from")
    message: Optional[str] = None
    received_at: Optional[str] = None
    mms_count: Optional[int] = None
    callback_data: Optional[str] = None
    price: Optional[Price] = None


class GetInboundMMSMessagesResponse(ResponseBase):
    results: Optional[List[Result]] = None



from enum import Enum
from typing import Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from pydantic import AnyHttpUrl, conlist, constr, validator

from infobip_channels.core.models import CamelCaseModel, UrlLengthValidatorMixin
from infobip_channels.whatsapp.models.body.core import MessageBody


class ButtonTypeEnum(str, Enum):
    REPLY = "REPLY"


class Footer(CamelCaseModel):
    text: constr(min_length=1, max_length=60)


class HeaderDocument(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["DOCUMENT"]
    media_url: AnyHttpUrl
    filename: Optional[constr(max_length=240)] = None

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderVideo(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["VIDEO"]
    media_url: AnyHttpUrl

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderImage(UrlLengthValidatorMixin, CamelCaseModel):
    type: Literal["IMAGE"]
    media_url: AnyHttpUrl

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class HeaderText(CamelCaseModel):
    type: Literal["TEXT"]
    text: constr(min_length=1, max_length=60)


class Button(CamelCaseModel):
    type: ButtonTypeEnum
    id: constr(min_length=1, max_length=256)
    title: constr(min_length=1, max_length=20)


class Action(CamelCaseModel):
    buttons: conlist(Button, min_items=1, max_items=3)


class Body(CamelCaseModel):
    text: constr(min_length=1, max_length=1024)


class Content(CamelCaseModel):
    body: Body
    action: Action
    header: Optional[Union[HeaderText, HeaderImage, HeaderDocument, HeaderVideo]] = None
    footer: Optional[Footer] = None


class ButtonsMessageBody(MessageBody):
    content: Content



from http import HTTPStatus
from typing import Any, Dict, Type, Union

import requests

from infobip_channels.core.channel import Channel
from infobip_channels.core.models import PostHeaders, QueryParameter, ResponseBase
from infobip_channels.mms.models.body.send_mms import MMSMessageBody
from infobip_channels.mms.models.query_parameters.get_inbound_mms_messages import (
    GetInboundMMSMessagesQueryParameters,
)
from infobip_channels.mms.models.query_parameters.get_mms_delivery_reports import (
    GetMMSDeliveryReportsQueryParameters,
)
from infobip_channels.mms.models.response.get_inbound_mms_messages import (
    GetInboundMMSMessagesResponse,
)
from infobip_channels.mms.models.response.get_mms_delivery_reports import (
    GetMMSDeliveryReportsResponse,
)
from infobip_channels.mms.models.response.send_mms import SendMMSResponse


class MMSChannel(Channel):
    """Class used for interaction with the Infobip's MMS API."""

    MMS_URL_TEMPLATE = "/mms/1/"

    @staticmethod
    def validate_query_parameter(
        parameter: Union[QueryParameter, Dict], parameter_type: Type[QueryParameter]
    ) -> QueryParameter:
        """
        Validate the query parameter by trying to instantiate the provided class.
        If the passed parameter is already of that type, just return it as is.

        :param parameter: Query parameter to validate
        :param parameter_type: Type of the query parameter
        :return: Class instance corresponding to the provided parameter type
        """
        return (
            parameter
            if isinstance(parameter, parameter_type)
            else parameter_type(**parameter)
        )

    def _get_custom_response_class(
        self,
        raw_response: Union[requests.Response, Any],
        response_class: Type[ResponseBase] = SendMMSResponse,
        *args,
        **kwargs
    ) -> Type[ResponseBase]:
        if raw_response.status_code in (
            HTTPStatus.OK,
            HTTPStatus.BAD_REQUEST,
            HTTPStatus.INTERNAL_SERVER_ERROR,
        ):
            return response_class

        raise ValueError

    def send_mms_message(
        self, message: Union[MMSMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a single MMS message to one destination address.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, MMSMessageBody)
        body, content_type = message.to_multipart()

        response = self._client.post(
            self.MMS_URL_TEMPLATE + "single",
            body,
            PostHeaders(
                content_type=content_type, authorization=self._client.auth.api_key
            ),
        )
        return self._construct_response(response, SendMMSResponse)

    def get_mms_delivery_reports(
        self, query_parameters: Union[GetMMSDeliveryReportsQueryParameters, Dict]
    ) -> Union[ResponseBase, Any]:
        """Use this API method to learn if and when the message has been delivered to
        the recipient. Each request will return a batch of delivery reports - only once.
        The following API request will return only new reports that arrived since the
        last API request.

        :param query_parameters: Query parameters to send with the request
        :return: Received response
        """
        query_parameters = self.validate_query_parameter(
            query_parameters, GetMMSDeliveryReportsQueryParameters
        )
        response = self._client.get(
            self.MMS_URL_TEMPLATE + "reports",
            params=query_parameters.dict(by_alias=True),
        )
        return self._construct_response(response, GetMMSDeliveryReportsResponse)

    def get_inbound_mms_messages(
        self, query_parameters: Union[GetInboundMMSMessagesQueryParameters, Dict]
    ) -> Union[ResponseBase, Any]:
        """Use this API method to fetch messages. Each request will return a batch of
        received messages - only once. The following API request will return only new
        messages that arrived since the last API request.

        :param query_parameters: Query parameters to send with the request
        :return: Received response
        """
        query_parameters = self.validate_query_parameter(
            query_parameters, GetInboundMMSMessagesQueryParameters
        )
        response = self._client.get(
            self.MMS_URL_TEMPLATE + "inbox/reports",
            params=query_parameters.dict(by_alias=True),
        )
        return self._construct_response(response, GetInboundMMSMessagesResponse)



from typing import Optional

from pydantic import constr

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.core import MessageBody


class Action(CamelCaseModel):
    catalog_id: str
    product_retailer_id: str


class Footer(CamelCaseModel):
    text: constr(min_length=1, max_length=60)


class Body(CamelCaseModel):
    text: constr(min_length=1, max_length=1024)


class Content(CamelCaseModel):
    action: Action
    body: Optional[Body] = None
    footer: Optional[Footer] = None


class ProductMessageBody(MessageBody):
    content: Content



from typing import List

from infobip_channels.core.models import ResponseBase
from infobip_channels.web_rtc.models.response.core import WebRtcResponseOKPayload


class GetApplicationsResponseOK(ResponseBase):
    list: List[WebRtcResponseOKPayload]



from typing import Optional

from infobip_channels.core.models import QueryParameter


class GetMMSDeliveryReportsQueryParameters(QueryParameter):
    bulk_id: Optional[str] = None
    message_id: Optional[str] = None
    limit: Optional[int] = None



from enum import Enum
from typing import List, Optional

from pydantic import conlist, constr, validator

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.action_sections import (
    SectionBase,
    SectionTitleValidatorMixin,
)
from infobip_channels.whatsapp.models.body.core import MessageBody


class HeaderTypeEnum(str, Enum):
    TEXT = "TEXT"


class Section(SectionBase):
    product_retailer_ids: List[str]


class Footer(CamelCaseModel):
    text: constr(min_length=1, max_length=60)


class Action(SectionTitleValidatorMixin, CamelCaseModel):
    catalog_id: str
    sections: conlist(Section, min_items=1, max_items=10)

    @validator("sections")
    def validate_section_titles(cls, sections: List[Section]) -> List[SectionBase]:
        super().validate_section_titles(sections)
        return sections


class Header(CamelCaseModel):
    type: HeaderTypeEnum
    text: constr(min_length=1, max_length=60)


class Body(CamelCaseModel):
    text: constr(min_length=1, max_length=1024)


class Content(CamelCaseModel):
    header: Header
    body: Body
    action: Action
    footer: Optional[Footer] = None


class MultiProductMessageBody(MessageBody):
    content: Content



from typing import Dict, Union

import requests

from infobip_channels.core.models import (
    Authentication,
    DeleteHeaders,
    GetHeaders,
    PostHeaders,
    PutHeaders,
    RequestHeaders,
)


class _HttpClient:
    """Default HTTP client used by the Infobip channels for making HTTP requests."""

    def __init__(
        self,
        auth: Authentication,
        post_headers: RequestHeaders = None,
        get_headers: RequestHeaders = None,
        put_headers: RequestHeaders = None,
        delete_headers: RequestHeaders = None,
    ):
        """Create an instance of the _HttpClient class with the provided
        authentication model instance. Headers can optionally be provided, otherwise
        default instances will be created. These headers will be used as defaults in
        the HTTP methods, unless new values are sent through method arguments.
        """
        self.auth = auth
        self.post_headers = post_headers or PostHeaders(authorization=self.auth.api_key)
        self.get_headers = get_headers or GetHeaders(authorization=self.auth.api_key)
        self.put_headers = put_headers or PutHeaders(authorization=self.auth.api_key)
        self.delete_headers = delete_headers or DeleteHeaders(
            authorization=self.auth.api_key
        )

    def post(
        self, endpoint: str, body: Union[Dict, bytes], headers: RequestHeaders = None
    ) -> requests.Response:
        """Send an HTTP post request to base_url + endpoint.

        :param endpoint: Which endpoint to hit
        :param body: Body to send with the request
        :param headers: Request headers
        :return: Received response
        """
        headers = headers or self.post_headers
        url = self.auth.base_url + endpoint

        if isinstance(body, dict):
            kwargs = {"json": body}
        else:
            kwargs = {"data": body}

        return requests.post(url=url, headers=headers.dict(by_alias=True), **kwargs)

    def get(
        self, endpoint: str, headers: RequestHeaders = None, params: Dict = None
    ) -> requests.Response:
        """Send an HTTP get request to base_url + endpoint.
        :param endpoint: Which endpoint to hit
        :param headers: Request headers
        :param params: Dictionary of query parameters
        :return: Received response
        """
        headers = headers or self.get_headers
        url = self.auth.base_url + endpoint

        return requests.get(url=url, headers=headers.dict(by_alias=True), params=params)

    def put(
        self, endpoint: str, body: Dict, headers: RequestHeaders = None
    ) -> requests.Response:
        """Send an HTTP put request to base_url + endpoint.

        :param endpoint: Which endpoint to hit
        :param headers: Request headers
        :param body: Body to send with the request
        :return: Received response
        """
        headers = headers or self.put_headers
        url = self.auth.base_url + endpoint

        return requests.put(url=url, json=body, headers=headers.dict(by_alias=True))

    def delete(
        self, endpoint: str, headers: RequestHeaders = None
    ) -> requests.Response:
        """Send an HTTP delete request to base_url + endpoint.

        :param endpoint: Which endpoint to hit
        :return: Received response
        """
        headers = headers or self.delete_headers
        url = self.auth.base_url + endpoint

        return requests.delete(url=url, headers=headers.dict(by_alias=True))



from typing import Dict, List, Optional

from infobip_channels.core.models import CamelCaseModel, ResponseBase


class ServiceException(CamelCaseModel):
    message_id: str
    text: str
    validation_errors: Optional[Dict[str, List[str]]] = None


class RequestError(CamelCaseModel):
    service_exception: ServiceException


class WebRtcResponseError(ResponseBase):
    request_error: RequestError


class AndroidServerKey(CamelCaseModel):
    fcm_server_key: str


class IosCertificate(CamelCaseModel):
    apns_certificate_file_name: str
    apns_certificate_password: str


class WebRtcResponseOKPayload(CamelCaseModel):
    id: str
    name: str
    description: str
    ios: Optional[IosCertificate] = None
    android: Optional[AndroidServerKey] = None
    app_to_app: bool
    app_to_conversations: bool
    app_to_phone: bool


class WebRtcResponseOK(WebRtcResponseOKPayload, ResponseBase):
    pass



from http import HTTPStatus
from typing import Any, Dict, Type, Union

import requests

from infobip_channels.core.channel import Channel
from infobip_channels.core.models import ResponseBase
from infobip_channels.whatsapp.models.body.audio_message import AudioMessageBody
from infobip_channels.whatsapp.models.body.buttons_message import ButtonsMessageBody
from infobip_channels.whatsapp.models.body.contact_message import ContactMessageBody
from infobip_channels.whatsapp.models.body.create_template import CreateTemplate
from infobip_channels.whatsapp.models.body.document_message import DocumentMessageBody
from infobip_channels.whatsapp.models.body.image_message import ImageMessageBody
from infobip_channels.whatsapp.models.body.list_message import ListMessageBody
from infobip_channels.whatsapp.models.body.location_message import LocationMessageBody
from infobip_channels.whatsapp.models.body.multi_product_message import (
    MultiProductMessageBody,
)
from infobip_channels.whatsapp.models.body.product_message import ProductMessageBody
from infobip_channels.whatsapp.models.body.sticker_message import StickerMessageBody
from infobip_channels.whatsapp.models.body.template_message import TemplateMessageBody
from infobip_channels.whatsapp.models.body.text_message import TextMessageBody
from infobip_channels.whatsapp.models.body.video_message import VideoMessageBody
from infobip_channels.whatsapp.models.path_parameters.manage_templates import (
    ManageTemplatesPathParameters,
)
from infobip_channels.whatsapp.models.response.core import (
    WhatsAppResponseError,
    WhatsAppResponseOK,
)
from infobip_channels.whatsapp.models.response.create_template import (
    CreateTemplateResponseOK,
)
from infobip_channels.whatsapp.models.response.get_templates import (
    GetTemplatesResponseOK,
)
from infobip_channels.whatsapp.models.response.template_message import (
    TemplateMessageResponseOK,
)


class WhatsAppChannel(Channel):
    """Class used for interaction with the Infobip's WhatsApp API."""

    SEND_MESSAGE_URL_TEMPLATE = "/whatsapp/1/message/"
    MANAGE_URL_TEMPLATE = "/whatsapp/1/senders/"

    def _get_custom_response_class(
        self,
        raw_response: Union[requests.Response, Any],
        response_ok_model: Type[ResponseBase] = WhatsAppResponseOK,
        *args,
        **kwargs
    ) -> Type[ResponseBase]:

        if raw_response.status_code in (HTTPStatus.OK, HTTPStatus.CREATED):
            return response_ok_model

        elif raw_response.status_code in (
            HTTPStatus.BAD_REQUEST,
            HTTPStatus.UNAUTHORIZED,
            HTTPStatus.FORBIDDEN,
            HTTPStatus.TOO_MANY_REQUESTS,
        ):
            return WhatsAppResponseError

        raise ValueError

    def send_template_message(
        self, message: Union[TemplateMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a single or multiple template messages to a one or more recipients.
        Template messages can be sent and delivered at anytime. Each template sent
        needs to be registered and pre-approved by WhatsApp.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, TemplateMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "template",
            message.dict(by_alias=True),
        )
        return self._construct_response(response, TemplateMessageResponseOK)

    def send_text_message(
        self, message: Union[TextMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a text message to a single recipient. Text messages can only be
        successfully delivered, if the recipient has contacted the business within the
        last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, TextMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "text", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_document_message(
        self, message: Union[DocumentMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a document to a single recipient. Document messages can only be
        successfully delivered, if the recipient has contacted the business within the
        last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, DocumentMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "document", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_image_message(
        self, message: Union[ImageMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        Send an image to a single recipient. Image messages can only be successfully
        delivered, if the recipient has contacted the business within the last 24
        hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, ImageMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "image", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_audio_message(
        self, message: Union[AudioMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send an audio to a single recipient. Audio messages can only be
        successfully delivered, if the recipient has contacted the business within
        the last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, AudioMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "audio", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_video_message(
        self, message: Union[VideoMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a video to a single recipient. Video messages can only be
        successfully delivered, if the recipient has contacted the business within
        the last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, VideoMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "video", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_sticker_message(
        self, message: Union[StickerMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a sticker to a single recipient. Sticker messages can only be
        successfully delivered, if the recipient has contacted the business within
        the last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, StickerMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "sticker", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_location_message(
        self, message: Union[LocationMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a location to a single recipient. Location messages can only be
        successfully delivered, if the recipient has contacted the business within
        the last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, LocationMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "location", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_contact_message(
        self, message: Union[ContactMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send a contact to a single recipient. Contact messages can only be
        successfully delivered, if the recipient has contacted the business within
        the last 24 hours, otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, ContactMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "contact", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def send_interactive_buttons_message(
        self, message: Union[ButtonsMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send an interactive buttons message to a single recipient. Interactive
        buttons messages can only be successfully delivered, if the recipient has
        contacted the business within the last 24 hours, otherwise template message
        should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, ButtonsMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "interactive/buttons",
            message.dict(by_alias=True),
        )
        return self._construct_response(response)

    def send_interactive_list_message(
        self, message: Union[ListMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send an interactive list message to a single recipient. Interactive list
        messages can only be successfully delivered, if the recipient has contacted
        the business within the last 24 hours, otherwise template message should be
        used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, ListMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "interactive/list",
            message.dict(by_alias=True),
        )
        return self._construct_response(response)

    def send_interactive_product_message(
        self, message: Union[ProductMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send an interactive product message to a single recipient. Interactive
        product messages can only be successfully delivered, if the recipient has
        contacted the business within the last 24 hours, otherwise template message
        should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, ProductMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "interactive/product",
            message.dict(by_alias=True),
        )
        return self._construct_response(response)

    def send_interactive_multi_product_message(
        self, message: Union[MultiProductMessageBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """Send an interactive multi-product message to a single recipient.
        Interactive multi-product messages can only be successfully delivered,
        if the recipient has contacted the business within the last 24 hours,
        otherwise template message should be used.

        :param message: Body of the message to send
        :return: Received response
        """
        message = self.validate_message_body(message, MultiProductMessageBody)

        response = self._client.post(
            self.SEND_MESSAGE_URL_TEMPLATE + "interactive/multi-product",
            message.dict(by_alias=True),
        )
        return self._construct_response(response)

    def get_templates(
        self, parameter: Union[ManageTemplatesPathParameters, Dict]
    ) -> Union[ResponseBase, Any]:
        """Get all the templates and their statuses for a given sender.

        :param parameter: Registered WhatsApp sender number. Must be in international
        format
        :return: Received response
        """
        path_parameter = self.validate_path_parameter(
            parameter, ManageTemplatesPathParameters
        )
        response = self._client.get(
            self.MANAGE_URL_TEMPLATE + path_parameter.sender + "/templates"
        )
        return self._construct_response(response, GetTemplatesResponseOK)

    def create_template(
        self,
        parameter: Union[ManageTemplatesPathParameters, Dict],
        message: Union[CreateTemplate, Dict],
    ) -> Union[ResponseBase, Any]:
        """Create WhatsApp template. Created template will be submitted for
        WhatsApp's review and approval. Once approved, template can be sent to
        end-users. Refer to template guidelines for additional info.

        :param parameter: Registered WhatsApp sender number. Must be in international
        format
        :param message: Body of the template to send
        :return: Received response
        """
        message = self.validate_message_body(message, CreateTemplate)
        path_parameter = self.validate_path_parameter(
            parameter, ManageTemplatesPathParameters
        )
        response = self._client.post(
            self.MANAGE_URL_TEMPLATE + path_parameter.sender + "/templates",
            message.dict(by_alias=True),
        )
        return self._construct_response(response, CreateTemplateResponseOK)



from datetime import datetime
from enum import Enum
from io import IOBase
from typing import List, Optional, Union

from pydantic import (
    AnyHttpUrl,
    Field,
    StrictBool,
    conint,
    conlist,
    constr,
    root_validator,
    validator,
)

from infobip_channels.core.models import (
    XML,
    CamelCaseModel,
    MessageBodyBase,
    MultipartMixin,
)

MINIMUM_DELIVERY_WINDOW_MINUTES = 60


class DaysEnum(str, Enum):
    MONDAY = "MONDAY"
    TUESDAY = "TUESDAY"
    WEDNESDAY = "WEDNESDAY"
    THURSDAY = "THURSDAY"
    FRIDAY = "FRIDAY"
    SATURDAY = "SATURDAY"
    SUNDAY = "SUNDAY"


class Time(CamelCaseModel):
    hour: conint(ge=0, le=23)
    minute: conint(ge=0, le=59)


class ExternallyHostedMedia(CamelCaseModel):
    content_type: str
    content_id: str
    content_url: AnyHttpUrl


class DeliveryTimeWindow(CamelCaseModel):
    days: conlist(DaysEnum, min_items=1)
    from_time: Optional[Time] = Field(alias="from", default=None)
    to: Optional[Time] = None

    @root_validator
    def validate_from_and_to(cls, values):
        if not values.get("from_time") and not values.get("to"):
            return values

        if values.get("from_time") and not values.get("to"):
            raise ValueError("If 'from_time' is set, 'to' has to be set also")

        if values.get("to") and not values.get("from_time"):
            raise ValueError("If 'to' is set, 'from_time' has to be set also")

        cls._validate_time_differences(values["from_time"], values["to"])

        return values

    @classmethod
    def _validate_time_differences(cls, from_time: Time, to_time: Time):
        from_time_in_minutes = from_time.hour * 60 + from_time.minute
        to_time_in_minutes = to_time.hour * 60 + to_time.minute

        if to_time_in_minutes - from_time_in_minutes < MINIMUM_DELIVERY_WINDOW_MINUTES:
            raise ValueError(
                f"Minimum of {MINIMUM_DELIVERY_WINDOW_MINUTES} minutes has to pass "
                f"between from and to delivery window times."
            )


class Head(CamelCaseModel):
    from_number: str = Field(alias="from")
    to: str
    id: Optional[str] = None
    subject: Optional[str] = None
    validity_period_minutes: Optional[int] = None
    callback_data: Optional[constr(max_length=200)] = None
    notify_url: Optional[AnyHttpUrl] = None
    send_at: Optional[Union[datetime, str]] = None
    intermediate_report: Optional[StrictBool] = False
    delivery_time_window: Optional[DeliveryTimeWindow] = None

    @validator("send_at")
    def convert_send_at_to_correct_format(cls, value):
        if not value:
            return

        if isinstance(value, str):
            value = datetime.fromisoformat(value)

        time_with_microseconds = value.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        return time_with_microseconds.split("Z")[0][:-3] + "Z"


class MMSMessageBody(MultipartMixin, MessageBodyBase):
    head: Head
    text: Optional[str] = None
    media: Optional[IOBase] = None
    externally_hosted_media: Optional[List[ExternallyHostedMedia]] = None
    smil: Optional[XML] = None

    class Config(CamelCaseModel.Config):
        arbitrary_types_allowed = True



from pydantic import AnyHttpUrl, validator

from infobip_channels.core.models import CamelCaseModel, UrlLengthValidatorMixin
from infobip_channels.whatsapp.models.body.core import MessageBody


class Content(UrlLengthValidatorMixin, CamelCaseModel):
    media_url: AnyHttpUrl

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class StickerMessageBody(MessageBody):
    content: Content



from typing import Optional

from pydantic import StrictBool, constr

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.core import MessageBody


class Content(CamelCaseModel):
    text: constr(min_length=1, max_length=4096)
    preview_url: Optional[StrictBool] = False


class TextMessageBody(MessageBody):
    content: Content



from typing import List, Optional

from pydantic import AnyHttpUrl

from infobip_channels.core.models import CamelCaseModel, ResponseBase


class Button(CamelCaseModel):
    text: str
    type: str
    phone_number: Optional[str] = None
    url: Optional[AnyHttpUrl] = None


class Header(CamelCaseModel):
    format: str
    text: Optional[str] = None


class Structure(CamelCaseModel):
    header: Optional[Header] = None
    body: str
    footer: Optional[str] = None
    type: str
    buttons: Optional[List[Button]] = None


class Template(CamelCaseModel):
    id: str
    business_account_id: int
    name: str
    language: str
    status: str
    category: str
    structure: Structure


class GetTemplatesResponseOK(ResponseBase):
    templates: List[Template]



from typing import Optional

from pydantic import AnyHttpUrl, constr, validator

from infobip_channels.core.models import CamelCaseModel, UrlLengthValidatorMixin
from infobip_channels.whatsapp.models.body.core import MessageBody


class Content(UrlLengthValidatorMixin, CamelCaseModel):
    media_url: AnyHttpUrl
    caption: Optional[constr(max_length=3000)] = None
    filename: Optional[constr(max_length=240)] = None

    @validator("media_url", pre=True)
    def validate_url_length(cls, value: str) -> str:
        return super().validate_url_length(value)


class DocumentMessageBody(MessageBody):
    content: Content



from http import HTTPStatus
from typing import Any, Dict, Type, Union

import requests

from infobip_channels.core.channel import Channel
from infobip_channels.core.models import ResponseBase
from infobip_channels.web_rtc.models.body.generate_token import GenerateTokenBody
from infobip_channels.web_rtc.models.body.save_application import SaveApplicationBody
from infobip_channels.web_rtc.models.body.update_application import (
    UpdateApplicationBody,
)
from infobip_channels.web_rtc.models.path_parameters.web_rtc_application import (
    WebRtcPathParameters,
)
from infobip_channels.web_rtc.models.response.core import (
    WebRtcResponseError,
    WebRtcResponseOK,
)
from infobip_channels.web_rtc.models.response.generate_token import (
    GenerateTokenResponseOK,
)
from infobip_channels.web_rtc.models.response.get_applications import (
    GetApplicationsResponseOK,
)


class WebRtcChannel(Channel):
    """Class used for interaction with the Infobip's WebRTC API."""

    WEB_RTC_URL_TEMPLATE = "/webrtc/1/"

    def _get_custom_response_class(
        self,
        response: Union[requests.Response, Any],
        response_ok_model: Type[ResponseBase] = WebRtcResponseOK,
        *args,
        **kwargs
    ) -> Type[ResponseBase]:

        if response.status_code == HTTPStatus.OK:
            return response_ok_model

        elif response.status_code in (
            HTTPStatus.BAD_REQUEST,
            HTTPStatus.UNAUTHORIZED,
        ):
            return WebRtcResponseError

        raise ValueError

    def generate_token(
        self, message: Union[GenerateTokenBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        This endpoint allows you to generate token for WebRTC channel.

        :param message: Body of request to send
        :return: Received response
        """
        message = self.validate_message_body(message, GenerateTokenBody)
        response = self._client.post(
            self.WEB_RTC_URL_TEMPLATE + "token", message.dict(by_alias=True)
        )
        return self._construct_response(response, GenerateTokenResponseOK)

    def save_application(
        self, message: Union[SaveApplicationBody, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        Create and configure a new WebRTC application.

        :param message: Body of request to send
        :return: Received response
        """
        message = self.validate_message_body(message, SaveApplicationBody)
        response = self._client.post(
            self.WEB_RTC_URL_TEMPLATE + "applications", message.dict(by_alias=True)
        )
        return self._construct_response(response)

    def get_applications(self) -> Union[ResponseBase, requests.Response, Any]:
        """
        List all applications for WebRTC channel.

        :return: Received response
        """
        response = self._client.get(self.WEB_RTC_URL_TEMPLATE + "applications")
        return self._construct_response(response, GetApplicationsResponseOK)

    def get_application(
        self, parameter: Union[WebRtcPathParameters, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        Get a single WebRTC application to see its configuration details.

        :param parameter: Application Id
        :return: Received response
        """
        path_parameter = self.validate_path_parameter(parameter, WebRtcPathParameters)
        response = self._client.get(
            self.WEB_RTC_URL_TEMPLATE + "applications/" + path_parameter.id
        )
        return self._construct_response(response)

    def delete_application(
        self, parameter: Union[WebRtcPathParameters, Dict]
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        Delete WebRTC application for a given id.

        :param parameter: Application Id
        :return: Received response
        """
        path_parameter = self.validate_path_parameter(parameter, WebRtcPathParameters)
        response = self._client.delete(
            self.WEB_RTC_URL_TEMPLATE + "applications/" + path_parameter.id
        )
        return self._construct_response(response)

    def update_application(
        self,
        parameter: Union[WebRtcPathParameters, Dict],
        message: Union[UpdateApplicationBody, Dict],
    ) -> Union[ResponseBase, requests.Response, Any]:
        """
        Change configuration details of your existing WebRTC application.

        :param parameter: Application Id
        :param message: Body of request to send
        :return: Received response
        """
        path_parameter = self.validate_path_parameter(parameter, WebRtcPathParameters)
        message = self.validate_message_body(message, UpdateApplicationBody)
        response = self._client.put(
            self.WEB_RTC_URL_TEMPLATE + "applications/" + path_parameter.id,
            message.dict(by_alias=True),
        )
        return self._construct_response(response)



from typing import Optional

from pydantic import constr

from infobip_channels.core.models import CamelCaseModel, MessageBodyBase


class AndroidServerKey(CamelCaseModel):
    fcm_server_key: constr(min_length=1)


class IosCertificates(CamelCaseModel):
    apns_certificate_file_name: constr(min_length=1)
    apns_certificate_file_content: constr(min_length=1)
    apns_certificate_password: Optional[str] = None


class ApplicationBody(MessageBodyBase):
    name: constr(min_length=1)
    description: Optional[constr(max_length=160)] = None
    ios: Optional[IosCertificates] = None
    android: Optional[AndroidServerKey] = None
    app_to_app: Optional[bool] = None
    app_to_conversations: Optional[bool] = None
    app_to_phone: Optional[bool] = None



from typing import List, Optional

from infobip_channels.core.models import CamelCaseModel, ResponseBase, ResponseStatus


class SendMMSResponseMessage(CamelCaseModel):
    to: Optional[str] = None
    status: ResponseStatus
    message_id: Optional[str] = None


class SendMMSResponse(ResponseBase):
    bulk_id: Optional[str] = None
    messages: List[SendMMSResponseMessage]
    error_message: Optional[str] = None



from typing import Optional

from pydantic import confloat, constr

from infobip_channels.core.models import CamelCaseModel
from infobip_channels.whatsapp.models.body.core import MessageBody


class Content(CamelCaseModel):
    latitude: confloat(ge=-90, le=90)
    longitude: confloat(ge=-180, le=180)
    name: Optional[constr(max_length=1000)] = None
    address: Optional[constr(max_length=1000)] = None


class LocationMessageBody(MessageBody):
    content: Content
